#!/usr/bin/env python2
from __future__ import with_statement
from __future__ import absolute_import
import os
import sys
import re
import subprocess

from urllib import urlencode
from io import open
from urlparse import urlparse
from urllib2 import urlopen

import argparse
import pafy
from .player import Player
from .process import kill_process_tree
from .process import toggle_process_tree

USAGE = u"""Usage: amp [SEARCH TERMS]
Pass search terms to YouTube and play the first result in a background process.
Call again with no arguments to stop playback.
"""

def main():
    pidfile = u"/tmp/amp.pid"

    parser = argparse.ArgumentParser(description=u"""Pass search terms to YouTube
    and play the first result in a background process.
    Call again with no arguments to pause or resume playback.""",
                                     prog=u'amp',)

    parser.add_argument(u'-v', action=u'store_true',
                        help=u'show the video as well')

    parser.add_argument(u'-k', action=u'store_true',
                        help=u'kill playback process')

    parser.add_argument(u'--verbose', action=u'store_true',
                        help=u'show verbose output')

    parser.add_argument(u'--version', action=u'version', version=u'%(prog)s 0.1.22-2')

    args = parser.parse_known_args()

    # If amp is called with no search terms, try to pause playback.
    if len(args[1]) == 0:
        try:
            with open(pidfile, u'r') as f:
                # Read the pidfile to get the pid of the process to kill.
                pid = int(f.read().strip())

                # If the -k flag is set, kill the process tree completely.
                if args[0].k:
                    try:
                        kill_process_tree(pid)
                        os.remove(pidfile)
                        print u'Killed playback process.'
                    except:
                        print u'Cannot find the amp process to kill.'


                # Otherwise, pause or resume the process tree.
                else:
                    try:
                        toggle_process_tree(pid)
                    except:
                        print u'pidfile invalid; was amp killed uncleanly?'

        except:
            # If the pidfile doesn't exist, then playback is not happening.
            # Print the usage message and return.
            parser.print_help()
        sys.exit(0)

    # Process the search terms, taking each argument and forming a string
    if args[1]:
        input = u" ".join(args[1])

    else:
        parser.print_help()
        sys.exit(0)

    query_string = urlencode({u"search_query": input})

    html_content = urlopen(u"https://www.youtube.com/results?" +
                           query_string)

    search_results = re.findall(ur'href=\"\/watch\?v=(.{11})',
                                html_content.read())

    url = u"https://www.youtube.com/watch?v=" + search_results[0]
    player = Player(pidfile, url,
                    show_video=args[0].v, verbose=args[0].verbose)
    player.start()

if __name__ == u'__main__':
    sys.exit(main())



#!/usr/bin/env python3
import os
import sys

import argparse
import pafy
from .player import Player
from .process import kill_process_tree
from .process import toggle_process_tree
from .util import get_search_results

USAGE = """Usage: amp [SEARCH TERMS]
Pass search terms to YouTube and play the first result in a background process.
Call again with no arguments to stop playback.
"""

PIDFILE = '/tmp/amp.pid'
INFOFILE = '/tmp/amp.info'

def main():

    opts = {}

    parser = argparse.ArgumentParser(description="""Pass search terms to YouTube
    and play the first result in a background process.
    Call again with no arguments to pause or resume playback.""",
                                     prog='amp',)

    parser.add_argument('-v', action='store_true',
                        help='show the video as well')
    parser.add_argument('-i', action='store_true',
                        help='print video info')
    parser.add_argument('-k', action='store_true',
                        help='kill playback process')
    parser.add_argument('--verbose', action='store_true',
                        help='show verbose output')
    parser.add_argument('--version', action='version',
                        version='%(prog)s 0.1.23-3')

    args = parser.parse_known_args()

    # Handle show video switch
    opts["vid"] = "auto" if args[0].v else "no"

    # Print the info of what's currently playing
    if (args[0].i):
        try:
            with open(INFOFILE, 'r') as f:
                print(f.read())
        except OSError:
            print('No info found.')

        sys.exit(0)

    # If amp is called with no search terms, try to pause playback.
    if len(args[1]) == 0:
        try:
            with open(PIDFILE, 'r') as f:
                # Read the pidfile to get the pid of the process to kill.
                pid = int(f.read().strip())

                # If the -k flag is set, kill the process tree completely.
                if args[0].k:
                    try:
                        kill_process_tree(pid)
                        os.remove(PIDFILE)
                        print('Killed playback process.')
                    except:
                        print('Cannot find the amp process to kill.')
                    return

                # Otherwise, pause or resume the process tree.
                else:
                    try:
                        toggle_process_tree(pid)
                    except:
                        print('pidfile invalid; was amp killed uncleanly?')

        except OSError:
            # If the pidfile doesn't exist, then playback is not happening.
            # Print the usage message and return.
            parser.print_help()
        sys.exit(0)

    # Process the search terms, taking each argument and forming a string
    if args[1]:
        input = " ".join(args[1])

    else:
        # If there's nothing to do, print help and exit.
        parser.print_help()
        sys.exit(0)

    search_results = get_search_results(input)
    url = 'https://www.youtube.com/watch?v=' + search_results[0]

    player = Player(url, show_video=args[0].v, verbose=args[0].verbose)
    player.start()

if __name__ == '__main__':
    sys.exit(main())



from __future__ import absolute_import
import psutil
import signal

def kill_process_tree(parent_pid):
    parent = psutil.Process(parent_pid)
    for child in parent.children(recursive=True):
        child.kill()
    parent.kill()

def toggle_process_tree(parent_pid):
    try:
        parent = psutil.Process(parent_pid)
    except: return
    if parent.status() == psutil.STATUS_STOPPED:
        for child in parent.children(recursive=True):
            child.resume()
        parent.resume()
        print u"Playback resumed."

    else:
        for child in parent.children(recursive=True):
            child.suspend()
        parent.suspend()
        print u"Playback paused. Type 'amp' again to resume."



u"""
Player daemon that handles asynchronous playback.
"""
from __future__ import with_statement
from __future__ import absolute_import
import sys
import os
import time
import atexit
import signal
import subprocess
import pafy
from .process import kill_process_tree

# Temporary change logging level until a bug with pafy is fixed to suppress
# unnecessary error
import logging
from io import open
logging.getLogger().setLevel(logging.ERROR)

class Player(object):

    u"""Daemon that controls the music player. Based on implementation by anon at
    http://jejik.com/articles/2007/02/a_simple_unix_linux_daemon_in_python/#c35
    """

    def __init__(self, pidfile, url, show_video=False, verbose=False):
        self.pidfile = pidfile
        self.url = url
        self.show_video = show_video
        self.verbose = verbose


    def print_info(self):
        u"""Prints video information and usage output to stdout"""

        video_data = pafy.new(self.url)
        print u"Now playing: " + video_data.title + u" [" + video_data.duration + u"]"

        # Handle passed-in options
        if self.verbose:
            print u"URL: " + self.url
            print u"Description: " + video_data.description
        if self.show_video:
            print u"Showing video in an external window."

    def daemonize(self):
        u"""Daemonize class. UNIX double fork mechanism."""

        try:
            pid = os.fork()
            if pid > 0:
                # exit first parent
                sys.exit(0)

        except OSError, err:
            sys.stderr.write(u'fork #1 failed: {0}\n'.format(err))
            sys.exit(1)

        # decouple from parent environment
        os.chdir(u'/')
        os.setsid()
        os.umask(0)

        # do second fork
        try:
            pid = os.fork()
            if pid > 0:

                # exit from second parent
                sys.exit(0)
        except OSError, err:
            sys.stderr.write(u'fork #2 failed: {0}\n'.format(err))
            sys.exit(1)

        # redirect standard file descriptors
        sys.stdout.flush()
        sys.stderr.flush()
        si = open(os.devnull, u'r')
        so = open(os.devnull, u'a+')
        se = open(os.devnull, u'a+')

        os.dup2(si.fileno(), sys.stdin.fileno())

        # write pidfile
        atexit.register(self.delpid)

        pid = unicode(os.getpid())
        with open(self.pidfile, u'w+') as f:
            f.write(pid + u'\n')

    def delpid(self):
        os.remove(self.pidfile)

    def start(self):
        try:
            with open(self.pidfile, u'r') as f:
                pid = int(f.read().strip())

        except IOError:
            pid = None

        if pid:
            print u"Stopping current song.."
            kill_process_tree(pid)
            self.delpid()

        self.print_info()

        self.daemonize()
        self.run()

    def stop(self):
        u"""Stop the daemon."""

        # Get the pid from the pidfile
        try:
            with open(self.pidfile, u'r') as pf:
                pid = int(pf.read().strip())

        except IOError:
            pid = None

        if not pid:
            message = u"pidfile {0} does not exist. " + \
                      u"Daemon not running?\n"
        sys.stderr.write(message.format(self.pidfile))
        return  # not an error in a restart

        # Try killing the daemon process
        try:
            while 1:
                os.kill(pid, signal.SIGTERM)
                time.sleep(0.1)

        except OSError, err:
            e = unicode(err.args)
            if e.find(u"No such process") > 0:
                if os.path.exists(self.pidfile):
                    os.remove(self.pidfile)
                else:
                    print unicode(err.args)
                    sys.exit(1)

    def restart(self):
        u"""Restart the daemon."""
        self.stop()
        self.start()

    def run(self):
        if self.show_video:
            subprocess_args = [u'mpv', self.url, u"--really-quiet", u"--fs"]
        else:
            subprocess_args = [u'mpv', self.url, u"--really-quiet", u"--no-video"]
        try:
            subprocess.call(subprocess_args)
        except OSError as e:
            if e.errno == 2:
                print("mpv cannot be found.")
                sys.exit(1)



import re
from urllib.parse import urlparse, urlencode
from urllib.request import urlopen

def get_search_results(input):
    """
    Takes search input and returns the list of URL's for that search,
    generated by YouTube.
    """
    query_string = urlencode({'search_query': input})
    html_content = urlopen('https://www.youtube.com/results?' +
                           query_string)
    search_results = re.findall(r'href=\"\/watch\?v=(.{11})',
                                html_content.read().decode())
    return search_results



import sys
from distutils.core import setup
from distutils.extension import Extension

if sys.version_info[0] == 2:
    print("Python 2 detected.  The Python 3 version is recommended.")
    base_dir = 'python2'
    pyversion = '2'
elif sys.version_info[0] == 3:
    print("Python 3 detected.")
    base_dir = 'python3'
    pyversion = '3'

setup(
    name='amp-player',
    version='0.1.23-' + pyversion,
    description='Asynchronous command-line YouTube interface',
    keywords=["music", "audio", "video", "stream", "youtube"],
    url='https://github.com/nivekuil/amp',
    download_url='https://github.com/nivekuil/amp/tarball/master',
    author='Kevin Liu',
    author_email='mail@nivekuil.com',
    license='GPL3',
    entry_points={'console_scripts': ['amp = amp.main:main']},
    packages = ['amp'],
    package_dir={
        'amp' : base_dir + '/amp'
    },
    install_requires=['pafy', 'psutil'],
    classifiers = [
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],

)
